//! Rhai wrappers for Git module functions
//!
//! This module provides Rhai wrappers for the functions in the Git module.

use rhai::{Engine, EvalAltResult, Array, Dynamic};
use crate::git::{self, GitError};

/// Register Git module functions with the Rhai engine
///
/// # Arguments
///
/// * `engine` - The Rhai engine to register the functions with
///
/// # Returns
///
/// * `Result<(), Box<EvalAltResult>>` - Ok if registration was successful, Err otherwise
pub fn register_git_module(engine: &mut Engine) -> Result<(), Box<EvalAltResult>> {
    // Register basic git functions
    engine.register_fn("git_clone", git_clone);
    engine.register_fn("git_list", git_list);
    engine.register_fn("git_update", git_update);
    engine.register_fn("git_update_force", git_update_force);
    engine.register_fn("git_update_commit", git_update_commit);
    engine.register_fn("git_update_commit_push", git_update_commit_push);
    engine.register_fn("git_has_changes", has_git_changes);
    engine.register_fn("git_find_repos", find_matching_repos);
    
    Ok(())
}

// Helper functions for error conversion
fn git_error_to_rhai_error<T>(result: Result<T, GitError>) -> Result<T, Box<EvalAltResult>> {
    result.map_err(|e| {
        Box::new(EvalAltResult::ErrorRuntime(
            format!("Git error: {}", e).into(),
            rhai::Position::NONE
        ))
    })
}

//
// Git Function Wrappers
//

/// Wrapper for git::git_clone
///
/// Clones a git repository to a standardized location in the user's home directory.
pub fn git_clone(url: &str) -> Result<String, Box<EvalAltResult>> {
    git_error_to_rhai_error(git::git_clone(url))
}

/// Wrapper for git::git_list
///
/// Lists all git repositories found in the user's ~/code directory.
pub fn git_list() -> Result<Array, Box<EvalAltResult>> {
    let repos = git_error_to_rhai_error(git::git_list())?;
    
    // Convert Vec<String> to Rhai Array
    let mut array = Array::new();
    for repo in repos {
        array.push(Dynamic::from(repo));
    }
    
    Ok(array)
}

/// Wrapper for git::has_git_changes
///
/// Checks if a git repository has uncommitted changes.
pub fn has_git_changes(repo_path: &str) -> Result<bool, Box<EvalAltResult>> {
    git_error_to_rhai_error(git::has_git_changes(repo_path))
}

/// Wrapper for git::find_matching_repos
///
/// Finds repositories matching a pattern or partial path.
pub fn find_matching_repos(pattern: &str) -> Result<Array, Box<EvalAltResult>> {
    let repos = git_error_to_rhai_error(git::find_matching_repos(pattern))?;
    
    // Convert Vec<String> to Rhai Array
    let mut array = Array::new();
    for repo in repos {
        array.push(Dynamic::from(repo));
    }
    
    Ok(array)
}

/// Wrapper for git::git_update
///
/// Updates a git repository by pulling the latest changes.
pub fn git_update(repo_path: &str) -> Result<String, Box<EvalAltResult>> {
    git_error_to_rhai_error(git::git_update(repo_path))
}

/// Wrapper for git::git_update_force
///
/// Force updates a git repository by discarding local changes and pulling the latest changes.
pub fn git_update_force(repo_path: &str) -> Result<String, Box<EvalAltResult>> {
    git_error_to_rhai_error(git::git_update_force(repo_path))
}

/// Wrapper for git::git_update_commit
///
/// Commits changes in a git repository and then updates it by pulling the latest changes.
pub fn git_update_commit(repo_path: &str, message: &str) -> Result<String, Box<EvalAltResult>> {
    git_error_to_rhai_error(git::git_update_commit(repo_path, message))
}

/// Wrapper for git::git_update_commit_push
///
/// Commits changes in a git repository and pushes them to the remote.
pub fn git_update_commit_push(repo_path: &str, message: &str) -> Result<String, Box<EvalAltResult>> {
    git_error_to_rhai_error(git::git_update_commit_push(repo_path, message))
}